#!/bin/sh

# ktrove.sh
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
######################################################################

OURNAME="KTrove"

IPKG_PROGRAM="ipkg"
OPIE_SH=/opt/QtPalmtop/bin/opie-sh

TROVE_BASE="/kmnt"
TROVE_LIST="/mnt/card|/mnt/cf"
CUR_TROVE="/mnt/card"
VERBOSE_MODE=0
TB_REFRESH=0

set -o noglob


err_string()
{
    local err=""

    case $1 in
        0)  err="None"
            ;;
        1)  err="Invalid Trove Path"
            ;;
        2)  err="Missing Parameter"
            ;;
        3)  err="This Program is already registered"
            ;;
        4)  err="This Program is not registered"
            ;;
        5)  err="This Program is activated"
            ;;
        6)  err="This Program is not activated"
            ;;
    esac

    echo "$OURNAME Error: $err"
}

do_verbose_msg()
{
    if [ "$VERBOSE_MODE" = "1" ] ; then
        echo "$OURNAME: $1"
    fi
}

print_usage()
{
    echo  "
Usage:
  $0 [options]
  A product of the BBC (no relation)
  THIS PROGRAM CAN MESS YOU UP.
  READ THE README.

  [options] is one or more of:
  -a <pgm>  Activate Program
  -c <pgm>  Check Program Status
  -d <pgm>  Deactivate Program
  -g        Use Opie-Sh GUI
  -h        This help message
  -l        List Available Packages
  -r <pgm>  Manage this Program
  -s        Gets the status of the trove.
  -t <path> Use A Trove at <path>
  -u <pgm>  Stop managing this program
  -v        be verbose
"
}

do_refresh_launcher()
{
}


list_packages()
{
    $IPKG_PROGRAM status | sed -ne "s/^Package:[[:space:]]\([a-zA-Z0-9.+-]*\)/\1/p" | sort | uniq
}

list_files()
{
    $IPKG_PROGRAM files $1 | sed -ne "s/^\///p"
}

list_status()
{
    local trovename="$1"

    if [ ! -e $trovename ] ; then
        err_string 1
        exit
    fi

    local fullkmntdir="$trovename$TROVE_BASE"
    set +o noglob
    local ctllist=`ls $fullkmntdir/*.kctl 2> /dev/null`
    set -o noglob
    local outmsg=""

    for fname in $ctllist; do
        local pkgname=`echo $fname | sed -ne "s/.*\/\([a-zA-Z0-9.+-]*\)\.kctl$/\1/p"`
        local kmntpkgdir="$fullkmntdir/$pkgname"

        outmsg="${outmsg}${pkgname}: "

        if [ ! -e $kmntpkgdir ] ; then
            outmsg="${outmsg}Missing Directory Structure\n"
        elif [ ! -e $fullkmntdir/.$pkgname.activated ] ; then
            outmsg="${outmsg}Not Activated\n"
        else
            outmsg="${outmsg}Activated\n"
        fi
    done

    echo $outmsg
}

check_package_status()
{
    local pkgname="$1"
    local trovename="$2"
    local fullkmntdir="$trovename$TROVE_BASE"
    local kmntpkgdir="$fullkmntdir/$pkgname"

    # Parameter Check
    #
    if [ "x$pkgname" = "x" ] || [ "x$trovename" = "x" ]; then
        err_string 2
        exit;
    fi

    if [ ! -e $trovename ] ; then
        err_string 1
        exit;
    fi

    if [ ! -e ${kmntpkgdir}.kctl ] || [ ! -e $kmntpkgdir ] ; then
        err_string 4
        exit
    fi

    if [ -e $fullkmntdir/.$pkgname.activated ] ; then
        echo "1"
    else
        echo "0"
    fi
}

register_package()
{
    local pkgname="$1"
    local trovename="$2"

    local fullkmntdir="$trovename$TROVE_BASE"
    local kmntpkgdir="$fullkmntdir/$pkgname"

    do_verbose_msg "Registering \"$pkgname\" in trove \"$trovename\""

    # Check Our Parameters
    #
    if [ "x$pkgname" = "x" ] || [ "x$trovename" = "x" ]; then
        err_string 2
        exit;
    fi

    if [ ! -e $trovename ] ; then
        err_string 1
        exit;
    fi

    if ! [ -e "$fullkmntdir" ] ; then
        do_verbose_msg "Creating $OURNAME root dir: $fullkmntdir"
        mkdir $fullkmntdir
    fi

    if [ -e $kmntpkgdir ] ; then
        err_string 3
        exit
    fi

    do_verbose_msg "Creating base dir: $kmntpkgdir"
    mkdir $kmntpkgdir

    local filelist=`list_files $1 | sed "/^\/lib\//D;/^\/usr\/lib\//D"`

    for thisfile in $filelist; do
        if [ -d $thisfile ] ; then
            if ! [ -e "$kmntpkgdir$thisfile" ] ; then
                do_verbose_msg "Creating dir $kmntpkgdir/$thisfile"
                mkdir "$kmntpkgdir$thisfile"
            fi
        else
            do_verbose_msg "Moving file from $thisfile to $kmntpkgdir"
            mv $thisfile $kmntpkgdir$thisfile
            do_verbose_msg "Adding moved file to control list"
            echo "$thisfile" >> $fullkmntdir/${pkgname}.kctl
        fi
    done

    TB_REFRESH=1
    do_verbose_msg "Done Registering \"$pkgname\""
}


unregister_package()
{
    local pkgname=$1
    local trovename="$2"
    local fullkmntdir="$trovename$TROVE_BASE"
    local kmntpkgdir="$fullkmntdir/$pkgname"

    do_verbose_msg "Unregistering \"$pkgname\" from trove \"$trovename\""

    # Parameter Check
    #
    if [ "x$pkgname" = "x" ] || [ "x$trovename" = "x" ]; then
        err_string 2
        exit;
    fi

    if [ ! -e $trovename ] ; then
        err_string 1
        exit;
    fi

    if [ ! -e ${kmntpkgdir}.kctl ] || [ ! -e $kmntpkgdir ] ; then
        err_string 4
        exit
    fi

    if [ -e $fullkmntdir/.$pkgname.activated ] ; then
        err_string 5
        exit
    fi

    local ctllist=`cat ${kmntpkgdir}.kctl`

    for thisfile in $ctllist; do
        do_verbose_msg "moving $thisfile back."
        mv $kmntpkgdir$thisfile $thisfile
    done

    do_verbose_msg "Removing $kmntpkgdir"
    rm -rf $kmntpkgdir
    do_verbose_msg "Removing ctl file"
    rm ${kmntpkgdir}.kctl

    TB_REFRESH=1
    do_verbose_msg "Done unregistering \"$pkgname\""
}

activate_package()
{
    local pkgname=$1
    local trovename="$2"
    local fullkmntdir="$trovename$TROVE_BASE"
    local kmntpkgdir="$fullkmntdir/$pkgname"

    do_verbose_msg "Activating \"$pkgname\" on trove \"$trovename\""

    # Parameter Check
    #
    if [ "x$pkgname" = "x" ] || [ "x$trovename" = "x" ]; then
        err_string 2
        exit;
    fi

    if [ ! -e $trovename ] ; then
        err_string 1
        exit;
    fi

    if [ ! -e ${kmntpkgdir}.kctl ] || [ ! -e $kmntpkgdir ] ; then
        err_string 4
        exit
    fi

    if [ -e $fullkmntdir/.$pkgname.activated ] ; then
        err_string 5
        exit
    fi

    local ctllist=`cat ${kmntpkgdir}.kctl`

    for thisfile in $ctllist; do
        do_verbose_msg "Linking to $thisfile"
        ln -s $kmntpkgdir$thisfile $thisfile
    done

    do_verbose_msg "Marking $pkgname activated"
    echo "23" > $fullkmntdir/.$pkgname.activated
    TB_REFRESH=1
    do_verbose_msg "Done Activating \"$pkgname\""
}

activate_all_packages()
{
    local trovename="$1"

    if [ ! -e $trovename ] ; then
        err_string 1
        exit
    fi

    local fullkmntdir="$trovename$TROVE_BASE"
    set +o noglob
    local ctllist=`ls $fullkmntdir/*.kctl 2> /dev/null`
    set -o noglob

    for fname in $ctllist; do
        local pkgname=`echo $fname | sed -ne "s/.*\/\([a-zA-Z0-9.+-]*\)\.kctl$/\1/p"`

        if [ ! -e $fullkmntdir/.$pkgname.activated ] ; then
            activate_package $pkgname $trovename
        fi
    done
}

deactivate_package()
{
    local pkgname=$1
    local trovename="$2"
    local fullkmntdir="$trovename$TROVE_BASE"
    local kmntpkgdir="$fullkmntdir/$pkgname"

    do_verbose_msg "Deactivating \"$pkgname\" on trove \"$trovename\""

    # Parameter Check
    #
    if [ "x$pkgname" = "x" ] || [ "x$trovename" = "x" ]; then
        err_string 2
        exit;
    fi

    if [ ! -e $trovename ] ; then
        err_string 1
        exit;
    fi

    if [ ! -e ${kmntpkgdir}.kctl ] || [ ! -e $kmntpkgdir ] ; then
        err_string 4
        exit
    fi

    if [ ! -e $fullkmntdir/.$pkgname.activated ] ; then
        err_string 6
        exit
    fi

    local ctllist=`cat ${kmntpkgdir}.kctl`

    for thisfile in $ctllist; do
        do_verbose_msg "Removing link from $thisfile"
        rm $thisfile
    done

    do_verbose_msg "Marking \"$pkgname\" inactive"
    rm $fullkmntdir/.$pkgname.activated
    TB_REFRESH=1
    do_verbose_msg "Done Deactivating \"$pkgname\""
}

deactivate_all_packages()
{
    local trovename="$1"

    if [ ! -e $trovename ] ; then
        err_string 1
        exit
    fi

    local fullkmntdir="$trovename$TROVE_BASE"
    set +o noglob
    local ctllist=`ls $fullkmntdir/*.kctl 2> /dev/null`
    set -o noglob

    for fname in $ctllist; do
        local pkgname=`echo $fname | sed -ne "s/.*\/\([a-zA-Z0-9.+-]*\)\.kctl$/\1/p"`

        if [ -e $fullkmntdir/.$pkgname.activated ] ; then
            deactivate_package $pkgname $trovename
        fi
    done
}


## Opie-sh Routines Begin Here

draw_back()
{
    local title="$1"
    local body="$2"

    echo "$body" | $OPIE_SH -f -t "$title" > /dev/null 2>&1 &
    SCREENCLEAN=$!
    sleep 1
}

erase_back()
{
    kill $SCREENCLEAN > /dev/null 2>&1
}

refresh_back()
{
    erase_back
    draw_back "$1" "$2"
}

construct_stdhdr()
{
    STDHDR="<div align="center"><BR>Current Trove Path: $CUR_TROVE</div><BR><HR><BR>"
}


showwaitscreen()
{
    echo "<div align=\"center\"><BR><H1>Working Hard.<BR>Please Wait.</H1></div>" | $OPIE_SH -f -t "Please Wait" > /dev/null 2>&1 &
    WAITSCREEN=$!
}

killwaitscreen()
{
    kill $WAITSCREEN > /dev/null 2>&1
}


gui_activate_package()
{
    # First, construct the list of packages that can be activated from this trove
    #
    local temp=`list_status $CUR_TROVE`
    local tlist=`echo -e $temp | sed -ne "s/^\(.*\):[[:space:]]Not Activated.*$/\1|/gp"`

    if ! [ "x$tlist" = "x" ]; then
        tlist="-ALL-|$tlist"
        local pkgname=`echo "$tlist" | tr "|" "\n" | $OPIE_SH -g -i -L "Program:" -t "Activate Which?" -l 2> /dev/null`

        if ! ( [ "$pkgname" = "-1" ] || [ "x$pkgname" = "x" ] ) ; then
            if [ "$pkgname" = "-ALL-" ]; then
                result=`activate_all_packages $CUR_TROVE`
            else
                result=`activate_package $pkgname $CUR_TROVE`
            fi

            if [ "x$result" = "x" ]; then
                refresh_back $OURNAME "${STDHDR}$pkgname is now activated."
            else
                refresh_back $OURNAME "${STDHDR}$result"
            fi
        else
            refresh_back $OURNAME "${STDHDR}"
        fi
    else
        refresh_back $OURNAME "${STDHDR}There are no programs available for activating in this trove."
    fi
}

gui_deactivate_package()
{
    # First, construct the list of packages that can be deactivated from this trove
    #
    local temp=`list_status $CUR_TROVE`
    local tlist=`echo -e $temp | sed -ne "s/^\(.*\):[[:space:]]Activated.*$/\1|/gp"`

    if ! [ "x$tlist" = "x" ]; then
        tlist="-ALL-|$tlist"
        local pkgname=`echo "$tlist" | tr "|" "\n" | $OPIE_SH -g -i -L "Program:" -t "Deactivate Which?" -l 2> /dev/null`

        if ! ( [ "$pkgname" = "-1" ] || [ "x$pkgname" = "x" ] ) ; then
            if [ "$pkgname" = "-ALL-" ]; then
                result=`deactivate_all_packages $CUR_TROVE`
            else
                result=`deactivate_package $pkgname $CUR_TROVE`
            fi

            if [ "x$result" = "x" ]; then
                refresh_back $OURNAME "${STDHDR}$pkgname is no longer activated."
            else
                refresh_back $OURNAME "${STDHDR}$result"
            fi
        else
            refresh_back $OURNAME "${STDHDR}"
        fi
    else
        refresh_back $OURNAME "${STDHDR}There are no programs available for deactivating in this trove."
    fi
}


gui_register_package()
{
    # First, construct the list of packages that can be activated from this trove
    #
    local pkglist=`list_packages $CUR_TROVE`
    local temp=`list_status $CUR_TROVE`
    local reglist=`echo -e $temp | sed -ne "s/^\(.*\):.*$/\1|/gp" | tr "|" "\n"`

    # Now, Weed out those packages that we have already registered
    #
    temp="$pkglist\n$reglist"
    temp=`echo -e "$temp" | sort | sed -ne '$!N; /^\(.*\)\n\1$/!P; /^\(.*\)\n\1$/d; D'`

    if ! [ "x$temp" = "x" ]; then
        local pkgname=`echo "$temp" | tr "|" "\n" | $OPIE_SH -g -i -L "Program:" -t "Register Which?" -l 2> /dev/null`

        if ! ( [ "$pkgname" = "-1" ] || [ "x$pkgname" = "x" ] ) ; then
            showwaitscreen
            result=`register_package $pkgname $CUR_TROVE`
            killwaitscreen

            if [ "x$result" = "x" ]; then
                refresh_back $OURNAME "${STDHDR}$pkgname is now registered. Be sure to activate it before using it."
            else
                refresh_back $OURNAME "${STDHDR}$result"
            fi
        else
            refresh_back $OURNAME "${STDHDR}"
        fi
    else
        refresh_back $OURNAME "${STDHDR}There are no programs available for registration."
    fi
}


gui_unregister_package()
{
    local temp=`list_status $CUR_TROVE`
    local reglist=`echo -e $temp | sed -ne "s/^\(.*\):.*$/\1|/gp" | tr "|" "\n"`

    if ! [ "x$temp" = "x" ]; then
        local pkgname=`echo "$reglist" | tr "|" "\n" | $OPIE_SH -g -i -L "Program:" -t "Unregister Which?" -l 2>/dev/null`

        if ! ( [ "$pkgname" = "-1" ] || [ "x$pkgname" = "x" ] ) ; then
            showwaitscreen
            result=`unregister_package $pkgname $CUR_TROVE`
            killwaitscreen

            if [ "x$result" = "x" ]; then
                refresh_back $OURNAME "${STDHDR}$pkgname is no longer registered."
            else
                refresh_back $OURNAME "${STDHDR}$result"
            fi
        else
            refresh_back $OURNAME "${STDHDR}"
        fi
    else
        refresh_back $OURNAME "${STDHDR}There are no programs available to unregister."
    fi
}


run_gui()
{
    local cmdlist="Quit|Change Trove|Activate Program|Deactivate Program|Register Program|Unregister Program|Get Program List|Get Trove Status|"
    local command=""

    construct_stdhdr
    draw_back $OURNAME "$STDHDR"

    while [ 1 ]; do
        command=`echo "$cmdlist" | tr "|" "\n" | $OPIE_SH -g -i -t "Do What?" -l 2>/dev/null`

        if [ "x$command" = "x" ]; then
            break
        fi

        case "$command" in
            "Quit")                 break
                                    ;;

            "Change Trove")         temp=`echo "$TROVE_LIST" | tr "|" "\n" | $OPIE_SH -g -i -E -L "Pick Trove Path:" -t "What Path?" -l 2>/dev/null`

                                    if [ ! -d $temp ]; then
                                        refresh_back $OURNAME "${STDHDR}Could Not Change Trove:<br>New path does not exist"
                                    else
                                        CUR_TROVE=$temp
                                        construct_stdhdr
                                        refresh_back $OURNAME "$STDHDR"
                                    fi
                                    ;;

            "Activate Program")     gui_activate_package
                                    ;;

            "Deactivate Program")   gui_deactivate_package
                                    ;;

            "Register Program")     gui_register_package
                                    ;;

            "Unregister Program")   gui_unregister_package
                                    ;;

            "Get Program List")     temp=`list_packages`
                                    tlist=`echo $temp | sed -ne "s/ /\<br\>/gp"`
                                    refresh_back $OURNAME "$STDHDR $tlist"
                                    ;;

            "Get Trove Status")     temp=`list_status $CUR_TROVE`

                                    if [ "x$temp" = "x" ]; then
                                        tlist="This Trove is not managing any programs."
                                    else
                                        tlist=`echo -e $temp | sed -ne "s/^\(.*\):[[:space:]]\(.*\)$/\<tr>\<td\>\1\<\/td\>\<td align=\"right\"\>\2\<\/td\>\<\/tr\>/gp"`
                                        tlist="<div align="center"><table width=\"100%\">$tlist</table></div>"
                                    fi

                                    refresh_back $OURNAME "$STDHDR $tlist"
                                    ;;
        esac
    done

    erase_back
}



######################################################################
# main
#

if [ "$1" = "-qcop" ]; then
    run_gui
else
    while getopts a:c:d:ghlr:st:u:v OPT;
    do
        case $OPT
        in
            a)  if [ "$OPTARG" = "-ALL-" ]; then
                    activate_all_packages $CUR_TROVE
                else
                    activate_package $OPTARG $CUR_TROVE
                fi
                ;;

            c)  check_package_status $OPTARG $CUR_TROVE
                ;;

            d)  if [ "$OPTARG" = "-ALL-" ]; then
                    deactivate_all_packages $CUR_TROVE
                else
                    deactivate_package $OPTARG $CUR_TROVE
                fi
                ;;

            g)  run_gui
                break
                ;;

            h)  print_usage
                break
                ;;

            l)  echo -e "`list_packages`"
                ;;

            r)  register_package $OPTARG $CUR_TROVE
                ;;

            s)  echo -e "`list_status $CUR_TROVE`"
                ;;

            t)  if [ ! -d $OPTARG ]; then
                    err_string 1
                else
                    CUR_TROVE=$OPTARG
                fi
                ;;

            u)  unregister_package $OPTARG $CUR_TROVE
                ;;

            v)  VERBOSE_MODE=1
                ;;

            \?) print_usage
                break
                ;;
        esac
    done
fi

if [ "$TB_REFRESH" = "1" ]; then
    /opt/QtPalmtop/bin/qcop QPE/TaskBar "reloadInputMethods()"
    /opt/QtPalmtop/bin/qcop QPE/System "linkChanged(QString)"
fi


